@@ -1,7 +1,3 @@
-/* This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
-
 fn main() {
     // Setup cfg aliases
     cfg_aliases::cfg_aliases! {
@@ -14,7 +10,7 @@ fn main() {
         vulkan: { all(not(wasm), any(windows, unix_wo_apple)) },
         metal: { all(not(wasm), apple) },
         dx12: { all(not(wasm), windows) },
-        dx11: { all(not(wasm), windows) },
-        gl: { any(wasm, unix_wo_apple) },
+        dx11: { all(false, not(wasm), windows) },
+        gl: { all(not(wasm), unix_wo_apple) },
     }
 }
